#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Transformations and Timers" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from wx import *
from wx.glcanvas import *


class myGLCanvas(GLCanvas):
    def __init__(self, parent):
        """
        Création du canevas recevant la modélisation 3D
        """
        GLCanvas.__init__(self, parent,-1)

        self.init = False
        self._angle = 30.
        self._cameraAngle = 0.
        self.context = GLContext(self)

        glutInit()
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        
        self.Bind(EVT_PAINT, self.OnPaint)
        self.Bind(EVT_SIZE, self.OnResize)
        self.Bind(EVT_KEY_DOWN, self.handleKeypress)
        
        CallLater(25, self.update, 0)

    
    def initRendering(self):
        glEnable(GL_DEPTH_TEST)


    def OnPaint(self, event):
        if not self.init :
            self.init = True

            self.SetCurrent(self.context)
            self.initRendering()

            self.OnResize(0)

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glRotatef(-self._cameraAngle, 0.0, 1.0, 0.0)
        glTranslatef(0.0, 0.0, -5.0)
        
        glPushMatrix()
        glTranslatef(0.0, -1.0, 0.0)
        glRotatef(self._angle, 0.0, 0.0, 1.0)
        
        glBegin(GL_QUADS)
        glVertex3f(-0.7, -0.5, 0.0)
        glVertex3f(0.7, -0.5, 0.0)
        glVertex3f(0.4, 0.5, 0.0)
        glVertex3f(-0.4, 0.5, 0.0)
        glEnd()
        glPopMatrix()

        glPushMatrix()
        glTranslatef(1.0, 1.0, 0.0)
        glRotatef(self._angle, 0.0, 1.0, 0.0)
        glScalef(0.7, 0.7, 0.7)

        glBegin(GL_TRIANGLES)
        glVertex3f(-0.5, -0.5, 0.0)
        glVertex3f(0.5, -0.5, 0.0)
        glVertex3f(-0.5, 0.0, 0.0)
        
        glVertex3f(-0.5, 0.0, 0.0)
        glVertex3f(0.5, -0.5, 0.0)
        glVertex3f(0.5, 0.0, 0.0)
        
        glVertex3f(-0.5, 0.0, 0.0)
        glVertex3f(0.5, 0.0, 0.0)
        glVertex3f(0.0, 0.5, 0.0)	
        glEnd()
        glPopMatrix()

        glPushMatrix()
        glTranslatef(-1.0, 1.0, 0.0)
        
        glRotatef(self._angle, 1.0, 2.0, 3.0)

        glBegin(GL_TRIANGLES)
        glVertex3f(0.5, -0.5, 0.0)
        glVertex3f(0.0, 0.5, 0.0)
        glVertex3f(-0.5, -0.5, 0.0)	
        glEnd()

        glPopMatrix()
        
        self.SwapBuffers()
        
          
    def OnResize(self, event):
        """
        Permet de dire à OpenGl comment convertir les coordonnées en valeurs
        pixel
        """
        if not self.init : return
        
        w, h = self.GetSize()

        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)

        self.Refresh()
        

    def update(self, value):
        self._angle += 2.
        if self._angle > 360. : self._angle -= 360.
        
        self.OnPaint(0)
        CallLater(25, self.update, 0)


    def handleKeypress(self, event):
        key = event.GetUnicodeKey()
        if key == 27 : exit()
            
        
class test(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, \
                          title = "Transformations and Timers - \
videotutorialsrock.com", \
                          size = (506, 432))
        aire = myGLCanvas(self)
        panel = wx.Panel(self, -1, size = (100, 432))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(aire, 1, wx.EXPAND)
        sizer.Add(panel, 0)
        
        self.SetSizer(sizer)

        self.Show()


if __name__ == '__main__' :
    app = wx.App()
    frame = test()
    frame.Show()
    app.MainLoop()
